import 'package:flutter/material.dart';
import 'package:adeptutils/src/collections/stack.dart';

class AdeptDrawer extends Drawer{
  GlobalKey<ScaffoldState> _scaffoldKey;
  State _state;
  int _selectedItemId;
  int get selectedItemId => _selectedItemId;
  ListStack<int> _visitedItemsId = new ListStack();
  ListStack<int> get visitedItemsId => _visitedItemsId;


  AdeptDrawer({
    Key key,
    GlobalKey<ScaffoldState> scaffoldKey,
    State state,
    int defaultSelectedItemId,
    ListStack<int> defaultVisitedItemsId,
    double elevation = 16.0,
    Widget child,
    String semanticLabel,
  }) : super(
    key: key,
    elevation: elevation,
    child: child,
    semanticLabel: semanticLabel,
  ){
    _scaffoldKey = scaffoldKey;
    _state = state;
    if(defaultVisitedItemsId != null && defaultVisitedItemsId.isNotEmpty) {
      _visitedItemsId = defaultVisitedItemsId;
      if(_visitedItemsId.isNotEmpty) _selectedItemId = _visitedItemsId.peek();
    } else selectItem(defaultSelectedItemId);
  }

  bool get isDrawerOpen => _scaffoldKey != null ? _scaffoldKey.currentState.isDrawerOpen : false;
  bool get isEndDrawerOpen => _scaffoldKey != null ? _scaffoldKey.currentState.isEndDrawerOpen : false;
  void openDrawer() => _scaffoldKey != null ? _scaffoldKey.currentState.openDrawer() : null;
  void openEndDrawer() => _scaffoldKey != null ? _scaffoldKey.currentState.openEndDrawer() : null;
  void closeDrawer(BuildContext context) {
    if(context == null){
      if(_state != null) context = _state.context; else
      if(_scaffoldKey != null) context = _scaffoldKey.currentContext;
    }
    if(context != null && isDrawerOpen) Navigator.pop(context);
  }

  void selectItem(int selectedItemId, {bool repeated = false, bool update = true}){
    if(selectedItemId == null) return;

    if(!repeated && _visitedItemsId.isNotEmpty) {
      int currentItemId = _visitedItemsId.peek();
      if(currentItemId == selectedItemId) return;
    }
    _visitedItemsId.push(_selectedItemId = selectedItemId);
    if(update && _state != null) _state.setState(() {});
  }

  void _onBackVisit(){
    if(_visitedItemsId.size > 1){
      _visitedItemsId.pop();
      _selectedItemId = _visitedItemsId.peek();
      if(_state != null) _state.setState(() {});
    }
  }

  /**
   * onBackPressed returns false to stop propagating the event, true in other cases
   */
  Future<bool> onBackPressed({BuildContext context}) async{
    if(isDrawerOpen) {
      closeDrawer(context);
      return false;
    }
    _onBackVisit();
    return true;
  }

}